/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn;

import net.fabricmc.api.ModInitializer;
import net.fabricmc.fabric.api.client.itemgroup.FabricItemGroupBuilder;
import net.fabricmc.fabric.api.registry.CommandRegistry;
import net.minecraft.class_1761;
import net.minecraft.class_1799;
import net.minecraft.class_2170;
import net.minecraft.class_2315;
import net.minecraft.class_2960;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import reborncore.common.recipes.RecipeCrafter;
import reborncore.common.registration.RegistrationManager;
import reborncore.common.util.Torus;
import techreborn.blockentity.fusionReactor.FusionControlComputerBlockEntity;
import techreborn.client.GuiHandler;
import techreborn.events.ModRegistry;
import techreborn.events.StackToolTipHandler;
import techreborn.init.*;
import techreborn.packets.ClientboundPackets;
import techreborn.packets.ServerboundPackets;
import techreborn.proxies.CommonProxy;
import techreborn.utils.BehaviorDispenseScrapbox;
import techreborn.utils.StackWIPHandler;
import techreborn.world.WorldGenerator;

public class TechReborn implements ModInitializer {

	public static final String MOD_ID = "techreborn";
	public static final Logger LOGGER = LogManager.getLogger(MOD_ID);
	public static CommonProxy proxy = new CommonProxy();
	public static TechReborn INSTANCE;

	public static class_1761 ITEMGROUP = FabricItemGroupBuilder.build(
			new class_2960("techreborn", "item_group"),
			() -> new class_1799(TRContent.NUKE));

	@Override
	public void onInitialize() {
		INSTANCE = this;
		@SuppressWarnings("unused")
		RegistrationManager registrationManager = new RegistrationManager("techreborn", getClass());

		// Done to force the class to load
		ModRecipes.GRINDER.getName();

		ClientboundPackets.init();
		ServerboundPackets.init();

		ModRegistry.setupShit();
		RecipeCrafter.soundHanlder = new ModSounds.SoundHandler();

		proxy.preInit();

		// Registers Chest Loot
		ModLoot.init();
		// MinecraftForge.EVENT_BUS.register(new ModLoot());
		// Client only init, needs to be done before parts system
		proxy.init();

		// WorldGen
		WorldGenerator.initBiomeFeatures();

		StackToolTipHandler.setup();
		StackWIPHandler.setup();

		// WorldGen
		// GameRegistry.registerWorldGenerator(new OilLakeGenerator(), 0);
		// Register Gui Handler
		// Event busses
//		MinecraftForge.EVENT_BUS.register(new BlockBreakHandler());
//		MinecraftForge.EVENT_BUS.register(new TRRecipeHandler());
//		MinecraftForge.EVENT_BUS.register(new MultiblockEventHandler());
//		MinecraftForge.EVENT_BUS.register(new MultiblockServerTickHandler());
//		MinecraftForge.EVENT_BUS.register(new TRTickHandler());

		GuiHandler.register();

		// Village stuff
//		if (ConfigTechReborn.enableRubberTreePlantation) {
//			VillagerRegistry.instance().registerVillageCreationHandler(new VillagePlantaionHandler());
//			//MapGenStructureIO.registerStructureComponent(VillageComponentRubberPlantaion.class, new ResourceLocation(MOD_ID, "rubberplantation").toString());
//			ModLootTables.CHESTS_RUBBER_PLANTATION.toString(); //Done to make it load, then it will be read from disk
//		}

		// Scrapbox
		if (BehaviorDispenseScrapbox.dispenseScrapboxes) {
			class_2315.method_10009(TRContent.SCRAP_BOX, new BehaviorDispenseScrapbox());
		}

		Torus.genSizeMap(FusionControlComputerBlockEntity.maxCoilSize);

		proxy.postInit();

		ModRecipes.postInit();

		CommandRegistry.INSTANCE.register(false, dispatcher -> dispatcher.register(class_2170.method_9247("recipe").executes(context -> {
			try {
				RecipeTemplate.generateFromInv(context.getSource().method_9207());
			} catch (Exception e){
				e.printStackTrace();
			}
			return 0;
		})));

		LOGGER.info("TechReborn setup done!");

	}

}
