/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.machine.tier3;

import reborncore.api.IListInfoProvider;
import reborncore.api.IToolDrop;
import reborncore.api.blockentity.InventoryProvider;
import reborncore.client.containerBuilder.IContainerProvider;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.containerBuilder.builder.ContainerBuilder;
import reborncore.common.registration.RebornRegister;
import reborncore.common.registration.config.ConfigRegistry;
import reborncore.common.blockentity.MachineBaseBlockEntity;
import reborncore.common.util.RebornInventory;
import reborncore.common.util.Tank;
import techreborn.TechReborn;
import techreborn.init.TRContent;
import techreborn.init.TRBlockEntities;
import techreborn.utils.FluidUtils;

import javax.annotation.Nullable;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2591;
import java.util.List;

@RebornRegister(TechReborn.MOD_ID)
public class QuantumTankBlockEntity extends MachineBaseBlockEntity
	implements InventoryProvider, IToolDrop, IListInfoProvider, IContainerProvider {

	@ConfigRegistry(config = "machines", category = "quantum_tank", key = "QuantumTankMaxStorage", comment = "Maximum amount of millibuckets a Quantum Tank can store")
	public static int maxStorage = Integer.MAX_VALUE;

	public Tank tank = new Tank("QuantumTankBlockEntity", maxStorage, this);
	public RebornInventory<QuantumTankBlockEntity> inventory = new RebornInventory<>(3, "QuantumTankBlockEntity", 64, this).withConfiguredAccess();

	public QuantumTankBlockEntity(){
		this(TRBlockEntities.QUANTUM_TANK);
	}

	public QuantumTankBlockEntity(class_2591<?> blockEntityTypeIn) {
		super(blockEntityTypeIn);
	}

	public void readWithoutCoords(final class_2487 tagCompound) {
		tank.read(tagCompound);
	}
	
	public class_2487 writeWithoutCoords(final class_2487 tagCompound) {
		tank.write(tagCompound);
		return tagCompound;
	}
	
	public class_1799 getDropWithNBT() {
		final class_2487 blockEntity = new class_2487();
		final class_1799 dropStack = TRContent.Machine.QUANTUM_TANK.getStack();
		this.writeWithoutCoords(blockEntity);
		dropStack.method_7980(new class_2487());
		dropStack.method_7969().method_10566("blockEntity", blockEntity);
		return dropStack;
	}
	
	// TileMachineBase
	@Override
	public void method_16896() {
		super.method_16896();
		if (!field_11863.field_9236) {
			if (FluidUtils.drainContainers(tank, inventory, 0, 1)
					|| FluidUtils.fillContainers(tank, inventory, 0, 1, tank.getFluid())) {
				this.syncWithAll();
			}
				
		}
	}
	
	@Override
	public boolean canBeUpgraded() {
		return false;
	}
	
	@Override
	public void method_11014(final class_2487 tagCompound) {
		super.method_11014(tagCompound);
		readWithoutCoords(tagCompound);
	}

	@Override
	public class_2487 method_11007(final class_2487 tagCompound) {
		super.method_11007(tagCompound);
		writeWithoutCoords(tagCompound);
		return tagCompound;
	}
	
	// ItemHandlerProvider
	@Override
	public RebornInventory<QuantumTankBlockEntity> getInventory() {
		return this.inventory;
	}
	
	// IToolDrop
	@Override
	public class_1799 getToolDrop(final class_1657 entityPlayer) {
		return this.getDropWithNBT();
	}
	
	// IListInfoProvider
	@Override
	public void addInfo(final List<class_2561> info, final boolean isReal, boolean hasData) {
		if (isReal | hasData) {
			if (!this.tank.getFluidInstance().isEmpty()) {
				info.add(new class_2585(this.tank.getFluidAmount() + " of " + this.tank.getFluid()));
			} else {
				info.add(new class_2585("Empty"));
			}
		}
		info.add(new class_2585("Capacity " + this.tank.getCapacity() + " mb"));
	}

	// IContainerProvider
	@Override
	public BuiltContainer createContainer(int syncID, final class_1657 player) {
		return new ContainerBuilder("quantumtank").player(player.field_7514).inventory().hotbar()
			.addInventory().blockEntity(this).fluidSlot(0, 80, 17).outputSlot(1, 80, 53)
			.sync(tank).addInventory().create(this, syncID);
	}

	@Nullable
	@Override
	public Tank getTank() {
		return tank;
	}
}
