/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.client.gui;

import net.minecraft.class_1657;
import net.minecraft.class_2338;
import net.minecraft.class_2680;
import reborncore.ClientProxy;
import reborncore.RebornCoreClient;
import reborncore.client.gui.builder.GuiBase;
import reborncore.client.gui.builder.widget.GuiButtonExtended;
import reborncore.client.gui.guibuilder.GuiBuilder;
import reborncore.client.multiblock.Multiblock;
import reborncore.client.multiblock.MultiblockRenderEvent;
import reborncore.client.multiblock.MultiblockSet;
import techreborn.init.TRContent;
import techreborn.blockentity.machine.multiblock.VacuumFreezerBlockEntity;

public class GuiVacuumFreezer extends GuiBase {

	VacuumFreezerBlockEntity blockEntity;

	public GuiVacuumFreezer(int syncID, final class_1657 player, final VacuumFreezerBlockEntity blockEntity) {
		super(player, blockEntity, blockEntity.createContainer(syncID, player));
		this.blockEntity = blockEntity;
	}

	@Override
	public void init() {
		super.init();
		RebornCoreClient.multiblockRenderEvent.setMultiblock(null);
	}
	
	@Override
	protected void method_2389(final float f, final int mouseX, final int mouseY) {
		super.method_2389(f, mouseX, mouseY);
		final GuiBase.Layer layer = GuiBase.Layer.BACKGROUND;

		// Battery slot
		drawSlot(8, 72, layer);
		// Input slots
		drawSlot(8, 72, layer);
		drawSlot(55, 45, layer);
		// Output slot
		drawOutputSlot(101, 45, layer);

		builder.drawJEIButton(this, 158, 5, layer);
		if (blockEntity.getMultiBlock()) {
			builder.drawHologramButton(this, 6, 4, mouseX, mouseY, layer);
		}
	}

	@Override
	protected void method_2388(final int mouseX, final int mouseY) {
		super.method_2388(mouseX, mouseY);
		final GuiBase.Layer layer = GuiBase.Layer.FOREGROUND;

		builder.drawProgressBar(this, blockEntity.getProgressScaled(100), 100, 76, 48, mouseX, mouseY, GuiBuilder.ProgressDirection.RIGHT, layer);
		if (blockEntity.getMultiBlock()) {
			addHologramButton(6, 4, 212, layer).clickHandler(this::onClick);
		} else {
			builder.drawMultiblockMissingBar(this, layer);
			addHologramButton(76, 56, 212, layer).clickHandler(this::onClick);
			builder.drawHologramButton(this, 76, 56, mouseX, mouseY, layer);
		}
		builder.drawMultiEnergyBar(this, 9, 19, (int) blockEntity.getEnergy(), (int) blockEntity.getMaxPower(), mouseX, mouseY, 0, layer);
	}

	public void onClick(GuiButtonExtended button, Double mouseX, Double mouseY){
		if (GuiBase.slotConfigType == SlotConfigType.NONE) {
			if (RebornCoreClient.multiblockRenderEvent.currentMultiblock == null) {
				{
					// This code here makes a basic multiblock and then sets to the selected one.
					final Multiblock multiblock = new Multiblock();
					class_2680 advancedCasing = TRContent.MachineBlocks.ADVANCED.getCasing().method_9564();
					class_2680 industrialCasing = TRContent.MachineBlocks.INDUSTRIAL.getCasing().method_9564();

					addComponent(0, -1, 0, advancedCasing, multiblock);
					addComponent(1, -1, 0, advancedCasing, multiblock);
					addComponent(0, -1, 1, advancedCasing, multiblock);
					addComponent(-1, -1, 0, advancedCasing, multiblock);
					addComponent(0, -1, -1, advancedCasing, multiblock);
					addComponent(-1, -1, -1, advancedCasing, multiblock);
					addComponent(-1, -1, 1, advancedCasing, multiblock);
					addComponent(1, -1, -1, advancedCasing, multiblock);
					addComponent(1, -1, 1, advancedCasing, multiblock);
					
					addComponent(1, -2, 0, industrialCasing, multiblock);
					addComponent(0, -2, 1, industrialCasing, multiblock);
					addComponent(-1, -2, 0, industrialCasing, multiblock);
					addComponent(0, -2, -1, industrialCasing, multiblock);
					addComponent(-1, -2, -1, industrialCasing, multiblock);
					addComponent(-1, -2, 1, industrialCasing, multiblock);
					addComponent(1, -2, -1, industrialCasing, multiblock);
					addComponent(1, -2, 1, industrialCasing, multiblock);

					addComponent(0, -3, 0, advancedCasing, multiblock);
					addComponent(1, -3, 0, advancedCasing, multiblock);
					addComponent(0, -3, 1, advancedCasing, multiblock);
					addComponent(-1, -3, 0, advancedCasing, multiblock);
					addComponent(0, -3, -1, advancedCasing, multiblock);
					addComponent(-1, -3, -1, advancedCasing, multiblock);
					addComponent(-1, -3, 1, advancedCasing, multiblock);
					addComponent(1, -3, -1, advancedCasing, multiblock);
					addComponent(1, -3, 1, advancedCasing, multiblock);
					
					final MultiblockSet set = new MultiblockSet(multiblock);
					RebornCoreClient.multiblockRenderEvent.setMultiblock(set);
					RebornCoreClient.multiblockRenderEvent.parent = blockEntity.method_11016();
					MultiblockRenderEvent.anchor = blockEntity.method_11016().method_10074();
				}
			} else {
				RebornCoreClient.multiblockRenderEvent.setMultiblock(null);
			}
		}
	}
	
	public void addComponent(final int x, final int y, final int z, final class_2680 blockState, final Multiblock multiblock) {
		multiblock.addComponent(new class_2338(x, y, z), blockState);
	}
}
