/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.init;

import reborncore.api.power.EnumPowerTier;
import reborncore.api.blockentity.IUpgrade;
import reborncore.common.powerSystem.PowerAcceptorBlockEntity;
import reborncore.common.registration.RebornRegister;
import reborncore.common.registration.config.ConfigRegistry;
import techreborn.TechReborn;
import techreborn.blocks.*;
import techreborn.blocks.cable.BlockCable;
import techreborn.blocks.generator.*;
import techreborn.blocks.lighting.BlockLamp;
import techreborn.blocks.storage.*;
import techreborn.blocks.tier0.BlockIronAlloyFurnace;
import techreborn.blocks.tier0.BlockIronFurnace;
import techreborn.blocks.tier1.*;
import techreborn.blocks.tier2.*;
import techreborn.blocks.tier3.*;
import techreborn.blocks.transformers.BlockHVTransformer;
import techreborn.blocks.transformers.BlockLVTransformer;
import techreborn.blocks.transformers.BlockMVTransformer;
import techreborn.config.ConfigTechReborn;
import techreborn.entities.EntityNukePrimed;
import techreborn.items.ItemDynamicCell;
import techreborn.items.ItemUpgrade;
import techreborn.utils.InitUtils;
import techreborn.blockentity.storage.AdjustableSUBlockEntity;

import javax.annotation.Nullable;
import net.minecraft.class_1299;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1935;
import net.minecraft.class_2248;
import java.util.Arrays;
import java.util.function.Function;

@RebornRegister(TechReborn.MOD_ID)
public class TRContent {
	
	// Misc Blocks
	public static class_2248 COMPUTER_CUBE;
	public static class_2248 NUKE;
	public static class_2248 REFINED_IRON_FENCE;
	public static class_2248 REINFORCED_GLASS;
	public static class_2248 RUBBER_LEAVES;
	public static class_2248 RUBBER_LOG;
	public static class_2248 RUBBER_PLANK_SLAB;
	public static class_2248 RUBBER_PLANK_STAIR;
	public static class_2248 RUBBER_PLANKS;
	public static class_2248 RUBBER_SAPLING;
	
	// Armor
	public static class_1792 CLOAKING_DEVICE;
	public static class_1792 LAPOTRONIC_ORBPACK;
	public static class_1792 LITHIUM_ION_BATPACK;

	// Battery
	public static class_1792 ENERGY_CRYSTAL;
	public static class_1792 LAPOTRON_CRYSTAL;
	public static class_1792 LAPOTRONIC_ORB;
	public static class_1792 LITHIUM_ION_BATTERY;
	public static class_1792 RED_CELL_BATTERY;

	// Tools
	public static class_1792 TREE_TAP;
	public static class_1792 WRENCH;

	public static class_1792 BASIC_CHAINSAW;
	public static class_1792 BASIC_DRILL;
	public static class_1792 BASIC_JACKHAMMER;
	public static class_1792 ELECTRIC_TREE_TAP;

	public static class_1792 ADVANCED_CHAINSAW;
	public static class_1792 ADVANCED_DRILL;
	public static class_1792 ADVANCED_JACKHAMMER;
	public static class_1792 ROCK_CUTTER;

	public static class_1792 INDUSTRIAL_CHAINSAW;
	public static class_1792 INDUSTRIAL_DRILL;
	public static class_1792 INDUSTRIAL_JACKHAMMER;
	public static class_1792 NANOSABER;
	public static class_1792 OMNI_TOOL;

	public static class_1792 DEBUG_TOOL;

	// Other
	public static class_1792 FREQUENCY_TRANSMITTER;
	public static class_1792 SCRAP_BOX;
	public static class_1792 MANUAL;
	public static ItemDynamicCell CELL;

	// Gem armor & tools
	@Nullable
	public static class_1792 BRONZE_SWORD;
	@Nullable
	public static class_1792 BRONZE_PICKAXE;
	@Nullable
	public static class_1792 BRONZE_SPADE;
	@Nullable
	public static class_1792 BRONZE_AXE;
	@Nullable
	public static class_1792 BRONZE_HOE;
	@Nullable
	public static class_1792 BRONZE_HELMET;
	@Nullable
	public static class_1792 BRONZE_CHESTPLATE;
	@Nullable
	public static class_1792 BRONZE_LEGGINGS;
	@Nullable
	public static class_1792 BRONZE_BOOTS;
	@Nullable
	public static class_1792 RUBY_SWORD;
	@Nullable
	public static class_1792 RUBY_PICKAXE;
	@Nullable
	public static class_1792 RUBY_SPADE;
	@Nullable
	public static class_1792 RUBY_AXE;
	@Nullable
	public static class_1792 RUBY_HOE;
	@Nullable
	public static class_1792 RUBY_HELMET;
	@Nullable
	public static class_1792 RUBY_CHESTPLATE;
	@Nullable
	public static class_1792 RUBY_LEGGINGS;
	@Nullable
	public static class_1792 RUBY_BOOTS;
	@Nullable
	public static class_1792 SAPPHIRE_SWORD;
	@Nullable
	public static class_1792 SAPPHIRE_PICKAXE;
	@Nullable
	public static class_1792 SAPPHIRE_SPADE;
	@Nullable
	public static class_1792 SAPPHIRE_AXE;
	@Nullable
	public static class_1792 SAPPHIRE_HOE;
	@Nullable
	public static class_1792 SAPPHIRE_HELMET;
	@Nullable
	public static class_1792 SAPPHIRE_CHESTPLATE;
	@Nullable
	public static class_1792 SAPPHIRE_LEGGINGS;
	@Nullable
	public static class_1792 SAPPHIRE_BOOTS;
	@Nullable
	public static class_1792 PERIDOT_SWORD;
	@Nullable
	public static class_1792 PERIDOT_PICKAXE;
	@Nullable
	public static class_1792 PERIDOT_SPADE;
	@Nullable
	public static class_1792 PERIDOT_AXE;
	@Nullable
	public static class_1792 PERIDOT_HOE;
	@Nullable
	public static class_1792 PERIDOT_HELMET;
	@Nullable
	public static class_1792 PERIDOT_CHESTPLATE;
	@Nullable
	public static class_1792 PERIDOT_LEGGINGS;
	@Nullable
	public static class_1792 PERIDOT_BOOTS;
		
	public enum SolarPanels implements class_1935 {
		BASIC(EnumPowerTier.MICRO, ConfigTechReborn.basicGenerationRateD, ConfigTechReborn.basicGenerationRateN), 
		ADVANCED(EnumPowerTier.LOW, ConfigTechReborn.advancedGenerationRateD, ConfigTechReborn.advancedGenerationRateN), 
		INDUSTRIAL(EnumPowerTier.MEDIUM, ConfigTechReborn.industrialGenerationRateD, ConfigTechReborn.industrialGenerationRateN),
		ULTIMATE(EnumPowerTier.HIGH, ConfigTechReborn.ultimateGenerationRateD, ConfigTechReborn.ultimateGenerationRateN), 
		QUANTUM(EnumPowerTier.EXTREME, ConfigTechReborn.quantumGenerationRateD, ConfigTechReborn.quantumGenerationRateN), 
		CREATIVE(EnumPowerTier.INFINITE, Integer.MAX_VALUE / 100, Integer.MAX_VALUE / 100);
		
		public final String name;
		public final class_2248 block;
		
		// Generation of EU during Day
		public int generationRateD;
		// Generation of EU during Night
		public int generationRateN;
		// Internal EU storage of solar panel
		public int internalCapacity;
		public final EnumPowerTier powerTier;
		
		SolarPanels(EnumPowerTier tier, int generationRateD, int generationRateN) {
			name = this.toString().toLowerCase();
			powerTier = tier;
			block = new BlockSolarPanel(this);
			this.generationRateD = generationRateD;
			this.generationRateN = generationRateN;
			// Buffer for 2 mins of work
			internalCapacity = generationRateD * 2_400;
			
			InitUtils.setup(block, name + "_solar_panel");
		}

		@Override
		public class_1792 method_8389() {
			return block.method_8389();
		}
	}

	public enum Cables implements class_1935 {
		COPPER(128, 12.0, true, EnumPowerTier.MEDIUM),
		TIN(32, 12.0, true, EnumPowerTier.LOW),
		GOLD(512, 12.0, true, EnumPowerTier.HIGH),
		HV(2048, 12.0, true, EnumPowerTier.EXTREME),
		GLASSFIBER(8192, 12.0, false, EnumPowerTier.INSANE),
		INSULATED_COPPER(128, 10.0, false, EnumPowerTier.MEDIUM),
		INSULATED_GOLD(512, 10.0, false, EnumPowerTier.HIGH),
		INSULATED_HV(2048, 10.0, false, EnumPowerTier.EXTREME),
		SUPERCONDUCTOR(Integer.MAX_VALUE / 4, 10.0, false, EnumPowerTier.INFINITE);
		

		public final String name;
		public final BlockCable block;

		public int transferRate;
		public int defaultTransferRate;
		public double cableThickness;
		public boolean canKill;
		public boolean defaultCanKill;
		public EnumPowerTier tier;
		
		
		Cables(int transferRate, double cableThickness, boolean canKill, EnumPowerTier tier) {
			name = this.toString().toLowerCase();
			this.transferRate = transferRate;
			this.defaultTransferRate = transferRate;
			this.cableThickness = cableThickness / 2;
			this.canKill = canKill;
			this.defaultCanKill = canKill;
			this.tier = tier;
			this.block = new BlockCable(this);
			InitUtils.setup(block, name + "_cable");
		}
		
		public class_1799 getStack() {
			return new class_1799(block);
		}
		
		@Override
		public class_1792 method_8389() {
			return block.method_8389();
		}
	}

	public enum Ores implements class_1935 {
		BAUXITE(6, 10, 10, 60), 
		CINNABAR(6, 3, 10, 126), 
		COPPER(8, 16, 20, 60), 
		GALENA(8, 16, 10, 60), 
		IRIDIUM(3, 3, 5, 60), 
		LEAD(6, 16, 20, 60), 
		PERIDOT(6, 3, 10, 250), 
		PYRITE(6, 3, 10, 126), 
		RUBY(6, 3, 10, 60), 
		SAPPHIRE(6, 3, 10, 60), 
		SHELDONITE(6, 3, 10, 250), 
		SILVER(6, 16, 20, 60), 
		SODALITE(6, 3, 10, 250),
		SPHALERITE(6, 3, 10, 126), 
		TIN(8, 16, 20, 60), 
		TUNGSTEN(6, 3, 10, 250);

		public final String name;
		public final class_2248 block;
		public final int veinSize;
		public final int veinsPerChunk;
		public final int minY;
		public final int maxY;

		Ores(int veinSize, int veinsPerChunk, int minY, int maxY) {
			name = this.toString().toLowerCase();
			block = new BlockOre();
			this.veinSize = veinSize;
			this.veinsPerChunk = veinsPerChunk;
			this.minY = minY;
			this.maxY = maxY;
			InitUtils.setup(block, name + "_ore");
		}

		@Override
		public class_1792 method_8389() {
			return block.method_8389();
		}
	}

	public enum StorageBlocks implements class_1935 {
		ALUMINUM, BRASS, BRONZE, CHROME, COPPER, ELECTRUM, INVAR, IRIDIUM, IRIDIUM_REINFORCED_STONE,
		IRIDIUM_REINFORCED_TUNGSTENSTEEL, LEAD, NICKEL, OSMIUM, PERIDOT, PLATINUM, RED_GARNET, REFINED_IRON, RUBY,
		SAPPHIRE, SILVER, STEEL, TIN, TITANIUM, TUNGSTEN, TUNGSTENSTEEL, YELLOW_GARNET, ZINC;

		public final String name;
		public final class_2248 block;

		StorageBlocks() {
			name = this.toString().toLowerCase();
			block = new BlockStorage();
			InitUtils.setup(block, name + "_storage_block");
		}

		@Override
		public class_1792 method_8389() {
			return block.method_8389();
		}
	}

	public enum MachineBlocks {
		BASIC(1020 / 25),
		ADVANCED(1700 / 25),
		INDUSTRIAL(2380 / 25);

		public final String name;
		public final class_2248 frame;
		public final class_2248 casing;

		MachineBlocks(int casingHeatCapacity) {
			name = this.toString().toLowerCase();
			frame = new BlockMachineFrame();
			InitUtils.setup(frame, name + "_machine_frame");
			casing = new BlockMachineCasing(casingHeatCapacity);
			InitUtils.setup(casing, name + "_machine_casing");
		}

		public class_2248 getFrame() {
			return frame;
		}

		public class_2248 getCasing() {
			return casing;
		}

		public static class_1935[] getCasings(){
			return Arrays.stream(MachineBlocks.values()).map((Function<MachineBlocks, class_1935>) machineBlocks -> () -> class_1792.method_7867(machineBlocks.casing)).toArray(class_1935[]::new);
		}
	}
	
	
	public enum Machine implements class_1935 {
		ALLOY_SMELTER(new BlockAlloySmelter()),
		ASSEMBLY_MACHINE(new BlockAssemblingMachine()),
		AUTO_CRAFTING_TABLE(new BlockAutoCraftingTable()),
		CHEMICAL_REACTOR(new BlockChemicalReactor()),
		COMPRESSOR(new BlockCompressor()),
		DISTILLATION_TOWER(new BlockDistillationTower()),
		EXTRACTOR(new BlockExtractor()),
		FLUID_REPLICATOR(new BlockFluidReplicator()),
		GRINDER(new BlockGrinder()),
		ELECTRIC_FURNACE(new BlockElectricFurnace()),
		IMPLOSION_COMPRESSOR(new BlockImplosionCompressor()),
		INDUSTRIAL_BLAST_FURNACE(new BlockIndustrialBlastFurnace()),
		INDUSTRIAL_CENTRIFUGE(new BlockIndustrialCentrifuge()),
		INDUSTRIAL_ELECTROLYZER(new BlockIndustrialElectrolyzer()),
		INDUSTRIAL_GRINDER(new BlockIndustrialGrinder()),
		INDUSTRIAL_SAWMILL(new BlockIndustrialSawmill()),
		IRON_ALLOY_FURNACE(new BlockIronAlloyFurnace()),
		IRON_FURNACE(new BlockIronFurnace()),
		MATTER_FABRICATOR(new BlockMatterFabricator()),
		RECYCLER(new BlockRecycler()),
		ROLLING_MACHINE(new BlockRollingMachine()),
		SCRAPBOXINATOR(new BlockScrapboxinator()),
		VACUUM_FREEZER(new BlockVacuumFreezer()),
		
		DIESEL_GENERATOR(new BlockDieselGenerator()),
		DRAGON_EGG_SYPHON(new BlockDragonEggSyphon()),
		FUSION_COIL(new BlockFusionCoil()),
		FUSION_CONTROL_COMPUTER(new BlockFusionControlComputer()),
		GAS_TURBINE(new BlockGasTurbine()),
		LIGHTNING_ROD(new BlockLightningRod()),
		MAGIC_ENERGY_ABSORBER (new BlockMagicEnergyAbsorber()),
		MAGIC_ENERGY_CONVERTER(new BlockMagicEnergyConverter()),
		PLASMA_GENERATOR(new BlockPlasmaGenerator()),
		SEMI_FLUID_GENERATOR(new BlockSemiFluidGenerator()),
		SOLID_FUEL_GENERATOR(new BlockSolidFuelGenerator()),
		THERMAL_GENERATOR(new BlockThermalGenerator()),
		WATER_MILL(new BlockWaterMill()),
		WIND_MILL(new BlockWindMill()),
		
		CREATIVE_QUANTUM_CHEST(new BlockCreativeQuantumChest()),
		CREATIVE_QUANTUM_TANK(new BlockCreativeQuantumTank()),
		DIGITAL_CHEST(new BlockDigitalChest()),
		QUANTUM_CHEST(new BlockQuantumChest()),
		QUANTUM_TANK(new BlockQuantumTank()),
		
		ADJUSTABLE_SU(new BlockAdjustableSU()),
		CHARGE_O_MAT(new BlockChargeOMat()),
		INTERDIMENSIONAL_SU(new BlockInterdimensionalSU()),
		LAPOTRONIC_SU(new BlockLapotronicSU()),
		LSU_STORAGE(new BlockLSUStorage()),
		LOW_VOLTAGE_SU(new BlockLowVoltageSU()),
		MEDIUM_VOLTAGE_SU(new BlockMediumVoltageSU()),
		HIGH_VOLTAGE_SU(new BlockHighVoltageSU()),
		LV_TRANSFORMER(new BlockLVTransformer()),
		MV_TRANSFORMER(new BlockMVTransformer()),
		HV_TRANSFORMER(new BlockHVTransformer()),
		
		ALARM(new BlockAlarm()),
		CHUNK_LOADER(new BlockChunkLoader()),
		LAMP_INCANDESCENT(new BlockLamp(14, 4, 10, 8)),
		LAMP_LED(new BlockLamp(15, 1, 1, 12)),
		PLAYER_DETECTOR(new BlockPlayerDetector());
		
		public final String name;
		public final class_2248 block;

		<B extends class_2248> Machine(B block) {
			this.name = this.toString().toLowerCase();
			this.block = block;
			InitUtils.setup(block, name);
		}
		
		public class_1799 getStack() {
			return new class_1799(block);
		}
		
		@Override
		public class_1792 method_8389() {
			return block.method_8389();
		}
	}
	
	public enum Dusts implements class_1935 {
		ALMANDINE, ALUMINUM, ANDESITE, ANDRADITE, ASHES, BASALT, BAUXITE, BRASS, BRONZE, CALCITE, CHARCOAL, CHROME,
		CINNABAR, CLAY, COAL, COPPER, DARK_ASHES, DIAMOND, DIORITE, ELECTRUM, EMERALD, ENDER_EYE, ENDER_PEARL, ENDSTONE,
		FLINT, GALENA, GOLD, GRANITE, GROSSULAR, INVAR, IRON, LAZURITE, LEAD, MAGNESIUM, MANGANESE, MARBLE, NETHERRACK,
		NICKEL, OBSIDIAN, OLIVINE, PERIDOT, PHOSPHOROUS, PLATINUM, PYRITE, PYROPE, RED_GARNET, RUBY, SALTPETER,
		SAPPHIRE, SAW, SILVER, SODALITE, SPESSARTINE, SPHALERITE, STEEL, SULFUR, TIN, TITANIUM, TUNGSTEN, UVAROVITE,
		YELLOW_GARNET, ZINC;

		public final String name;
		public final class_1792 item;

		Dusts() {
			name = this.toString().toLowerCase();
			item = new class_1792(new class_1792.class_1793().method_7892(TechReborn.ITEMGROUP));
			InitUtils.setup(item, name + "_dust");
		}

		public class_1799 getStack() {
			return new class_1799(item);
		}

		public class_1799 getStack(int amount) {
			return new class_1799(item, amount);
		}

		@Override
		public class_1792 method_8389() {
			return item;
		}
	}

	public enum SmallDusts implements class_1935 {
		ALMANDINE, ALUMINUM, ANDESITE, ANDRADITE, ASHES, BASALT, BAUXITE, BRASS, BRONZE, CALCITE, CHARCOAL, CHROME,
		CINNABAR, CLAY, COAL, COPPER, DARK_ASHES, DIAMOND, DIORITE, ELECTRUM, EMERALD, ENDER_EYE, ENDER_PEARL, ENDSTONE,
		FLINT, GALENA, GLOWSTONE, GOLD, GRANITE, GROSSULAR, INVAR, IRON, LAZURITE, LEAD, MAGNESIUM, MANGANESE, MARBLE,
		NETHERRACK, NICKEL, OBSIDIAN, OLIVINE, PERIDOT, PHOSPHOROUS, PLATINUM, PYRITE, PYROPE, REDSTONE, RED_GARNET,
		RUBY, SALTPETER, SAPPHIRE, SAW, SILVER, SODALITE, SPESSARTINE, SPHALERITE, STEEL, SULFUR, TIN, TITANIUM,
		TUNGSTEN, UVAROVITE, YELLOW_GARNET, ZINC;

		public final String name;
		public final class_1792 item;

		SmallDusts() {
			name = this.toString().toLowerCase();
			item = new class_1792(new class_1792.class_1793().method_7892(TechReborn.ITEMGROUP));
			InitUtils.setup(item, name + "_small_dust");
		}

		public class_1799 getStack() {
			return new class_1799(item);
		}

		public class_1799 getStack(int amount) {
			return new class_1799(item, amount);
		}

		@Override
		public class_1792 method_8389() {
			return item;
		}
	}

	public enum Gems implements class_1935 {
		PERIDOT, RED_GARNET, RUBY, SAPPHIRE, YELLOW_GARNET;

		public final String name;
		public final class_1792 item;

		Gems() {
			name = this.toString().toLowerCase();
			item = new class_1792(new class_1792.class_1793().method_7892(TechReborn.ITEMGROUP));
			InitUtils.setup(item, name + "_gem");
		}

		public class_1799 getStack() {
			return new class_1799(item);
		}

		public class_1799 getStack(int amount) {
			return new class_1799(item, amount);
		}

		@Override
		public class_1792 method_8389() {
			return item;
		}
	}

	public enum Ingots implements class_1935 {
		ADVANCED_ALLOY, ALUMINUM, BRASS, BRONZE, CHROME, COPPER, ELECTRUM, HOT_TUNGSTENSTEEL, INVAR, IRIDIUM_ALLOY, IRIDIUM,
		LEAD, MIXED_METAL, NICKEL, PLATINUM, REFINED_IRON, SILVER, STEEL, TIN, TITANIUM, TUNGSTEN, TUNGSTENSTEEL, ZINC;

		public final String name;
		public final class_1792 item;

		Ingots() {
			name = this.toString().toLowerCase();
			item = new class_1792(new class_1792.class_1793().method_7892(TechReborn.ITEMGROUP));
			InitUtils.setup(item, name + "_ingot");
		}

		public class_1799 getStack() {
			return new class_1799(item);
		}

		public class_1799 getStack(int amount) {
			return new class_1799(item, amount);
		}

		@Override
		public class_1792 method_8389() {
			return item;
		}
	}

	public enum Nuggets implements class_1935 {
		ALUMINUM, BRASS, BRONZE, CHROME, COPPER, DIAMOND, ELECTRUM, HOT_TUNGSTENSTEEL, INVAR, IRIDIUM, LEAD, NICKEL,
		PLATINUM, REFINED_IRON, SILVER, STEEL, TIN, TITANIUM, TUNGSTEN, TUNGSTENSTEEL, ZINC;

		public final String name;
		public final class_1792 item;

		Nuggets() {
			name = this.toString().toLowerCase();
			item = new class_1792(new class_1792.class_1793().method_7892(TechReborn.ITEMGROUP));
			InitUtils.setup(item, name + "_nugget");
		}

		public class_1799 getStack() {
			return new class_1799(item);
		}

		public class_1799 getStack(int amount) {
			return new class_1799(item, amount);
		}

		@Override
		public class_1792 method_8389() {
			return item;
		}
	}

	public enum Parts implements class_1935 {
		CARBON_FIBER,
		CARBON_MESH,

		ELECTRONIC_CIRCUIT,
		ADVANCED_CIRCUIT,
		INDUSTRIAL_CIRCUIT,

		MACHINE_PARTS,
		DIGITAL_DISPLAY,

		DATA_STORAGE_CORE,
		DATA_STORAGE_CHIP,
		ENERGY_FLOW_CHIP,
		SUPERCONDUCTOR,

		DIAMOND_SAW_BLADE,
		DIAMOND_GRINDING_HEAD,
		TUNGSTEN_GRINDING_HEAD,

		CUPRONICKEL_HEATING_COIL,
		KANTHAL_HEATING_COIL,
		NICHROME_HEATING_COIL,

		NEUTRON_REFLECTOR,
		THICK_NEUTRON_REFLECTOR,
		IRIDIUM_NEUTRON_REFLECTOR

		//java vars can't start with numbers, so these get suffixes
		, WATER_COOLANT_CELL_10K,
		WATER_COOLANT_CELL_30K,
		WATER_COOLANT_CELL_60K,
		
		HELIUM_COOLANT_CELL_60K,
		HELIUM_COOLANT_CELL_180K,
		HELIUM_COOLANT_CELL_360K,

		NAK_COOLANT_CELL_60K,
		NAK_COOLANT_CELL_180K,
		NAK_COOLANT_CELL_360K,

		RUBBER,
		SAP,
		SCRAP,
		UU_MATTER;

		public final String name;
		public final class_1792 item;

		Parts() {
			name = this.toString().toLowerCase();
			item = new class_1792(new class_1792.class_1793().method_7892(TechReborn.ITEMGROUP));
			InitUtils.setup(item, name);
		}

		public class_1799 getStack() {
			return new class_1799(item);
		}

		public class_1799 getStack(int amount) {
			return new class_1799(item, amount);
		}

		@Override
		public class_1792 method_8389() {
			return item;
		}
	}

	public enum Plates implements class_1935 {
		ADVANCED_ALLOY, ALUMINUM, BRASS, BRONZE, CARBON, CHROME, COAL, COPPER, DIAMOND, ELECTRUM, EMERALD, GOLD, INVAR,
		IRIDIUM_ALLOY, IRIDIUM, IRON, LAPIS, LAZURITE, LEAD, MAGNALIUM, NICKEL, OBSIDIAN, PERIDOT, PLATINUM, RED_GARNET,
		REDSTONE, REFINED_IRON, RUBY, SAPPHIRE, SILICON, SILVER, STEEL, TIN, TITANIUM, TUNGSTEN, TUNGSTENSTEEL, WOOD,
		YELLOW_GARNET, ZINC;

		public final String name;
		public final class_1792 item;

		Plates() {
			name = this.toString().toLowerCase();
			item = new class_1792(new class_1792.class_1793().method_7892(TechReborn.ITEMGROUP));
			InitUtils.setup(item, name + "_plate");
		}

		public class_1799 getStack() {
			return new class_1799(item);
		}

		public class_1799 getStack(int amount) {
			return new class_1799(item, amount);
		}

		@Override
		public class_1792 method_8389() {
			return item;
		}
	}

	@ConfigRegistry(config = "items", category = "upgrades", key = "overclcoker_speed", comment = "Overclocker behavior speed multipiler")
	public static double overclockerSpeed = 0.25;

	@ConfigRegistry(config = "items", category = "upgrades", key = "overclcoker_power", comment = "Overclocker behavior power multipiler")
	public static double overclockerPower = 0.75;

	@ConfigRegistry(config = "items", category = "upgrades", key = "energy_storage", comment = "Energy storage behavior extra power")
	public static double energyStoragePower = 40_000;

	public enum Upgrades implements class_1935 {
		OVERCLOCKER((blockEntity, handler, stack) -> {
			PowerAcceptorBlockEntity powerAcceptor = null;
			if (blockEntity instanceof PowerAcceptorBlockEntity) {
				powerAcceptor = (PowerAcceptorBlockEntity) blockEntity;
			}
			handler.addSpeedMulti(overclockerSpeed);
			handler.addPowerMulti(overclockerPower);
			if (powerAcceptor != null) {
				powerAcceptor.extraPowerInput += powerAcceptor.getMaxInput();
				powerAcceptor.extraPowerStorage += powerAcceptor.getBaseMaxPower();
			}
		}),
		TRANSFORMER((blockEntity, handler, stack) -> {
			PowerAcceptorBlockEntity powerAcceptor = null;
			if (blockEntity instanceof PowerAcceptorBlockEntity) {
				powerAcceptor = (PowerAcceptorBlockEntity) blockEntity;
			}
			if (powerAcceptor != null) {
				powerAcceptor.extraTier += 1;
			}
		}),
		ENERGY_STORAGE((blockEntity, handler, stack) -> {
			PowerAcceptorBlockEntity powerAcceptor = null;
			if (blockEntity instanceof PowerAcceptorBlockEntity) {
				powerAcceptor = (PowerAcceptorBlockEntity) blockEntity;
			}
			if (powerAcceptor != null) {
				powerAcceptor.extraPowerStorage += energyStoragePower;
			}
		}),
		SUPERCONDUCTOR((blockEntity, handler, stack) -> {
			AdjustableSUBlockEntity aesu = null;
			if (blockEntity instanceof AdjustableSUBlockEntity) {
				aesu = (AdjustableSUBlockEntity) blockEntity;
			}
			if (aesu != null) {
				aesu.superconductors++;
			}
		});

		public String name;
		public class_1792 item;

		Upgrades(IUpgrade upgrade) {
			name = this.toString().toLowerCase();
			item = new ItemUpgrade(name, upgrade);
			InitUtils.setup(item, name + "_upgrade");
		}

		@Override
		public class_1792 method_8389() {
			return item;
		}
	}
	
	public static class_1299<EntityNukePrimed> ENTITY_NUKE;
}
