/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.items;

import net.minecraft.class_1761;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_2371;
import net.minecraft.class_2378;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2960;
import net.minecraft.class_3611;
import net.minecraft.class_3612;
import org.apache.commons.lang3.Validate;
import reborncore.common.fluid.FluidUtil;
import reborncore.common.fluid.container.ItemFluidInfo;
import reborncore.common.util.ItemNBTHelper;
import techreborn.TechReborn;
import techreborn.init.TRContent;
import techreborn.utils.FluidUtils;

/**
 * Created by modmuss50 on 17/05/2016.
 */
public class ItemDynamicCell extends class_1792 implements ItemFluidInfo {

	public ItemDynamicCell() {
		super(new class_1792.class_1793().method_7889(1).method_7892(TechReborn.ITEMGROUP));
	}

	@Override
	public void method_7850(class_1761 tab, class_2371<class_1799> subItems) {
		if (!method_7877(tab)) {
			return;
		}
		subItems.add(getEmptyCell(1));
		for (class_3611 fluid : FluidUtils.getAllFluids()) {
			if(fluid.method_15793(fluid.method_15785())){
				subItems.add(getCellWithFluid(fluid));
			}
		}
	}

	@Override
	public class_2561 method_7864(class_1799 itemStack) {
		class_3611 fluid = getFluid(itemStack);
		if(fluid != class_3612.field_15906){
			//TODO use translation keys for fluid and the cell https://fabric.asie.pl/wiki/tutorial:lang?s[]=translation might be useful
			return new class_2585(FluidUtil.getFluidName(fluid) + " Cell");
		}
		return super.method_7864(itemStack);
	}

	public static class_1799 getCellWithFluid(class_3611 fluid, int stackSize) {
		Validate.notNull(fluid);
		class_1799 stack = new class_1799(TRContent.CELL);
		ItemNBTHelper.getNBT(stack).method_10582("fluid", class_2378.field_11154.method_10221(fluid).toString());
		stack.method_7939(stackSize);
		return stack;
	}

	public static class_1799 getEmptyCell(int amount) {
		return new class_1799(TRContent.CELL, amount);
	}

	public static class_1799 getCellWithFluid(class_3611 fluid) {
		return getCellWithFluid(fluid, 1);
	}

	@Override
	public class_1799 getEmpty() {
		return new class_1799(this);
	}

	@Override
	public class_1799 getFull(class_3611 fluid) {
		return getCellWithFluid(fluid);
	}

	@Override
	public class_3611 getFluid(class_1799 itemStack) {
		class_2487 tag = itemStack.method_7969();
		if(tag != null && tag.method_10545("fluid")){
			return class_2378.field_11154.method_10223(new class_2960(tag.method_10558("fluid")));
		}
		return class_3612.field_15906;
	}
}
